=== modified file 'NEWS'
--- NEWS	2008-07-31 05:41:42 +0000
+++ NEWS	2008-08-15 16:24:50 +0000
@@ -1,9 +1,14 @@
 What's changed in loggerhead?
 =============================
 
-1.6 [unreleased]
+1.6 [15Aug2008]
 ----------------
 
+    - Download a diffs between revisions. (Martin Albisetti)
+
+    - Modified templates to make loggerhead's theme easier to
+      change. (Paul Hummer)
+
     - Default sqlite interface is now sqlite3. (Robert Collins)
 
     - New ninja theme sponsored by Canonical (Martin Albisetti)
@@ -13,12 +18,15 @@
     - Remove the .py extension requiered by the Debian Policy.
       (Jelmer Vernooij)
 
+    - New startup script serve-branches will serve Loggerhead without
+      the need of configuration, and allow you to browse through directories
+      and branches. (Michael Hudson)
+
     - Loggerhead is no longer a TurboGears application, but rather a
       WSGI application built using Paste (see http://wsgi.org/ and
-      http://pythonpaste.org/ for more about WSGI and Paste). XXX more.
+      http://pythonpaste.org/ for more about WSGI and Paste).
 
-    - zpt.cleaner_urls bug 138021 ("loggerhead should generate links
-      based on revision numbers and file paths").  XXX more.
+    - URLs now use revision numbers instead of revision ids (Martin Albisetti)
 
     - The scripts no longer insist on Python 2.4 -- loggerhead works
       fine with 2.5.

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2008-08-05 05:17:47 +0000
+++ loggerhead/apps/branch.py	2008-08-12 19:24:49 +0000
@@ -17,6 +17,7 @@
 from loggerhead.controllers.atom_ui import AtomUI
 from loggerhead.controllers.download_ui import DownloadUI
 from loggerhead.controllers.search_ui import SearchUI
+from loggerhead.controllers.diff_ui import DiffUI
 from loggerhead.history import History
 from loggerhead import util
 
@@ -78,6 +79,7 @@
         'download': DownloadUI,
         'atom': AtomUI,
         'search': SearchUI,
+        'diff': DiffUI,
         }
 
     def last_updated(self):

=== added file 'loggerhead/controllers/diff_ui.py'
--- loggerhead/controllers/diff_ui.py	1970-01-01 00:00:00 +0000
+++ loggerhead/controllers/diff_ui.py	2008-08-14 02:49:52 +0000
@@ -0,0 +1,99 @@
+# Copyright (C) 2008  Canonical Ltd.
+#                     (Authored by Martin Albisetti <argentina@gmail.com>)
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+from cStringIO import StringIO
+import logging
+import time
+import sys
+
+from paste import httpexceptions
+from paste.request import path_info_pop
+
+from loggerhead import history
+from loggerhead import util
+from loggerhead.templatefunctions import templatefunctions
+
+import bzrlib
+from bzrlib import branch
+from bzrlib.diff import show_diff_trees
+
+
+log = logging.getLogger("loggerhead.controllers")
+
+class DiffUI(object):
+    """
+
+    Class to output a diff for a single file or revisions.
+    """
+    
+    def __init__(self, branch, history):
+        self._branch = branch
+        self._history = history
+        self.log = history.log
+
+    
+    def __call__(self, environ, start_response):
+        # /diff/<rev_id>/<rev_id>
+        """
+        Default method called from /diff URL.
+        """
+
+        z = time.time()
+        
+        args = []
+        while 1:
+            arg = path_info_pop(environ)
+            if arg is None:
+                break
+            args.append(arg)
+
+        revid_from = args[0]
+        # Convert a revno to a revid if we get a revno
+        revid_from = self._history.fix_revid(revid_from)
+        change = self._history.get_changes([revid_from])[0]
+
+        if len(args) is 2:
+            revid_to = self._history.fix_revid(args[1])
+        else:
+            revid_to = change.parents[0].revid 
+
+
+        repo = self._branch.branch.repository
+        revtree1 = repo.revision_tree(revid_from)
+        revtree2 = repo.revision_tree(revid_to)
+        
+        diff_content_stream = StringIO()
+        show_diff_trees(revtree1, revtree2, diff_content_stream, 
+                        old_label='', new_label='')
+
+        content = diff_content_stream.getvalue()
+
+        self.log.info('/diff %r:%r in %r secs' % (revid_from, revid_to, 
+                                                  time.time() - z))
+
+        revno1 = self._history.get_revno(revid_from)
+        revno2 = self._history.get_revno(revid_to)
+        filename = '%s_%s.diff' % (revno1, revno2)
+        headers = [
+            ('Content-Type', 'application/octet-stream'),
+            ('Content-Length', len(content)),
+            ('Content-Disposition', 'attachment; filename=%s'%(filename,)),
+            ]
+        start_response('200 OK', headers)
+        return [content]
+

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2008-08-04 21:22:16 +0000
+++ loggerhead/controllers/directory_ui.py	2008-08-06 21:25:24 +0000
@@ -30,8 +30,12 @@
         self.parity = parity
         self.branch = branch
         if branch is not None:
-            self.last_change =  datetime.datetime.fromtimestamp(
-                branch.repository.get_revision(branch.last_revision()).timestamp)
+            # If a branch is empty, bzr raises an exception when trying this
+            try:
+                self.last_change =  datetime.datetime.fromtimestamp(
+                    branch.repository.get_revision(branch.last_revision()).timestamp)
+            except:
+                self.last_change = None
 
 class DirectoryUI(TemplatedBranchView):
     """

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2008-07-26 17:53:10 +0000
+++ loggerhead/history.py	2008-08-12 16:36:23 +0000
@@ -257,13 +257,19 @@
             revid = parents[0]
 
     def get_short_revision_history_by_fileid(self, file_id):
-        # wow.  is this really the only way we can get this list?  by
-        # man-handling the weave store directly? :-0
         # FIXME: would be awesome if we could get, for a folder, the list of
-        # revisions where items within that folder changed.
-        possible_keys = [(file_id, revid) for revid in self._full_history]
-        existing_keys = self._branch.repository.texts.get_parent_map(possible_keys)
-        return [revid for _, revid in existing_keys.iterkeys()]
+        # revisions where items within that folder changed.i
+        try:
+            # FIXME: Workaround for bzr versions prior to 1.6b3. 
+            # Remove me eventually pretty please  :)
+            w = self._branch.repository.weave_store.get_weave(file_id, self._branch.repository.get_transaction())
+            w_revids = w.versions() 
+            revids = [r for r in self._full_history if r in w_revids] 
+        except AttributeError:
+            possible_keys = [(file_id, revid) for revid in self._full_history]
+            existing_keys = self._branch.repository.texts.get_parent_map(possible_keys)
+            revids = [revid for _, revid in existing_keys.iterkeys()]
+        return revids
 
     def get_revision_history_since(self, revid_list, date):
         # if a user asks for revisions starting at 01-sep, they mean inclusive,

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2008-08-05 06:17:12 +0000
+++ loggerhead/static/css/global.css	2008-08-13 17:33:41 +0000
@@ -120,7 +120,7 @@
 #finderBox {
 	width:300px;
 	float:right;
-	padding:0px;
+	padding: 10px 0px 0px 0px;
 }
 #finderBox label {
 	float:left;

=== modified file 'loggerhead/static/javascript/custom.js'
--- loggerhead/static/javascript/custom.js	2008-07-22 00:40:03 +0000
+++ loggerhead/static/javascript/custom.js	2008-08-14 20:31:07 +0000
@@ -4,6 +4,7 @@
 window.addEvent('domready', function() 
 {
     var search_box = $('q');
+    search_box.removeEvents();
     search_box.addEvents(
     {
         keyup: function()
@@ -14,10 +15,10 @@
             }
             else
             {
-	            if (null != global_timeout_id)
-	            {
-	                clearTimeout(global_timeout_id);
-	            }
+                if (null != global_timeout_id)
+                {
+                    clearTimeout(global_timeout_id);
+                }
                     global_timeout_id = setTimeout('$("q").fireEvent("search",$("q").value)',200);
             }
         },
@@ -25,21 +26,25 @@
         search: function(query)
         {
             url = global_path + 'search?query=' + query;
-	
+
             if ($defined(global_search_request))
             {
                 global_search_request.cancel();
             }
-            global_search_request = new Request({'url':url,'method':'get','onComplete':cool_search});
+            global_search_request = new Request({'url':url,'method':'get','onComplete':function(response)
+              {
+                 cool_search(response,query);
+             }});
+
             global_search_request.send('');
-	        var posicion = search_box.getPosition();
-	        var size     = search_box.getSize();
+            var posicion = search_box.getPosition();
+            var size     = search_box.getSize();
 
-	        $('search_terms').setStyle('position','absolute');
-	        $('search_terms').setStyle('left',posicion.x);
-	        $('search_terms').setStyle('top',posicion.y + size.y);
-	        $('search_terms').setStyle('display','block');
-	        $('search_terms').set('html','Loading...');
+            $('search_terms').setStyle('position','absolute');
+            $('search_terms').setStyle('left',posicion.x);
+            $('search_terms').setStyle('top',posicion.y + size.y);
+            $('search_terms').setStyle('display','block');
+            $('search_terms').set('html','Loading...');
 
             new Request({'url':url,'method':'get','onComplete':cool_search}).send('');
 
@@ -48,15 +53,19 @@
     });
 });
 
-function cool_search(response)
+function cool_search(response, query)
 {
 	var posicion = $('q').getPosition();
 	var size     = $('q').getSize();
-	$('search_terms').set('html',response);
-	$('search_terms').setStyle('display','block');
-	$('search_terms').setStyle('position','absolute');
-	$('search_terms').setStyle('left',posicion.x);
-	$('search_terms').setStyle('top',posicion.y + size.y);
+    var current_query = $('q').get('value');
+    if (current_query == query)
+    {
+	    $('search_terms').set('html',response);
+	    $('search_terms').setStyle('display','block');
+	    $('search_terms').setStyle('position','absolute');
+	    $('search_terms').setStyle('left',posicion.x);
+	    $('search_terms').setStyle('top',posicion.y + size.y);
+    }
 }
 
 function hide_search()

=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2008-07-26 14:52:44 +0000
+++ loggerhead/templatefunctions.py	2008-08-13 19:51:08 +0000
@@ -74,3 +74,17 @@
     return _pt('modified-file-link-log').expand(
         url=url, entry=entry, item=item,
         **templatefunctions)
+
+@templatefunc
+def search_box(branch, navigation):
+    return _pt('search-box').expand(branch=branch, navigation=navigation,
+        **templatefunctions)
+
+@templatefunc
+def feed_link(branch, url):
+    return _pt('feed-link').expand(branch=branch, url=url, **templatefunctions)
+
+@templatefunc
+def menu(branch, url, fileview_active=False):
+    return _pt('menu').expand(branch=branch, url=url,
+        fileview_active=fileview_active, **templatefunctions)

=== added file 'loggerhead/templates/feed-link.pt'
--- loggerhead/templates/feed-link.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/feed-link.pt	2008-08-13 15:18:17 +0000
@@ -0,0 +1,6 @@
+<tal:feed-link>
+  <a tal:attributes="href python:url(['/atom']);
+                     title string:RSS feed for ${branch/friendly_name}">
+    <img tal:attributes="src python:branch.static_url('/static/images/ico_rss.gif')" alt="RSS" class="rssfeed"/>
+  </a>
+</tal:feed-link>

=== modified file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	2008-08-05 06:21:09 +0000
+++ loggerhead/templates/inventory.pt	2008-08-06 04:20:07 +0000
@@ -64,7 +64,7 @@
                <a tal:attributes="href python:url(['/files', change.revno], file_id=file.file_id)" 
                                       tal:content="file/filename" class="link"></a>
                </td>
-               <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno], filter_file_id=file.file_id);
+               <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
                                                    title string:Show revision ${file/change/revno}"
                                    tal:content="file/change/revno"></a>
                </td>
@@ -87,7 +87,7 @@
                <a tal:attributes="href python:url(['/annotate', change.revno], file_id=file.file_id)" 
                                       tal:content="file/filename" class="link"></a>
                </td>
-               <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno], filter_file_id=file.file_id);
+               <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
                                                    title string:Show revision ${file/change/revno}"
                                    tal:content="file/change/revno"></a>
                </td>
@@ -116,7 +116,7 @@
 
             <a tal:attributes="href python:url(['/annotate', change.revno], file_id=file.file_id)"
                                    tal:content="file/filename" class="link"></a></td>
-            <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno], filter_file_id=file.file_id);
+            <td class="date"><a tal:attributes="href python:url(['/revision', file.change.revno]);
                                                 title string:Show revision ${file/change/revno}"
                                 tal:content="file/change/revno"></a>
             </td>

=== modified file 'loggerhead/templates/macros.pt'
--- loggerhead/templates/macros.pt	2008-08-04 03:42:01 +0000
+++ loggerhead/templates/macros.pt	2008-08-13 19:51:08 +0000
@@ -21,41 +21,25 @@
 
 <body>
     <!-- Loggerhead Content Area -->
-    <ul id="menuTabs">
-        <form tal:attributes="action python:branch.url('/changes', start_revid=getattr(navigation, 'start_revid', None), 
-                                                                   file_id=getattr(navigation, 'file_id', None))">
-      	  <div id="finderBox">
-            <label>search:</label> <input type="search" name="q" id="q" autocomplete="off" onblur="hide_search();" /> 
-            <div><a tal:attributes="href python:url(['/atom']);
-                                    title string:RSS feed for ${branch/friendly_name}"> 
-                 <img tal:attributes="src python:branch.static_url('/static/images/ico_rss.gif')" alt="RSS" class="rssfeed"/></a>
-            </div>
-          </div>
-        </form>
+      <div id="finderBox">
+         <tal:search-box define="navigation navigation"
+                        replace="structure python:search_box(branch,
+                                 navigation)" />
+        <div>
+          <tal:feed-link replace="structure python:feed_link(branch, url)" />
+        </div>
+      </div>
 
-        <tal:block tal:condition="not:fileview_active">
-        <li><a tal:attributes="href python:url('/changes', clear=1);
-                               title string:Changes;
-                               id string:on">Changes</a></li>
-        <li><a tal:attributes="href python:url('/files', clear=1);
-                               title string:Files">Files</a></li>
-        </tal:block>
-        <tal:block tal:condition="fileview_active">
-        <li><a tal:attributes="href python:url('/changes', clear=1);
-                               title string:Changes">Changes</a></li>
-        <li><a tal:attributes="href python:url('/files', clear=1);
-                               title string:Files;
-                               id string:on">Files</a></li>
-        </tal:block>
-      </ul>
+      <tal:menu define="fileview_active fileview_active"
+                replace="structure python:menu(branch, url, fileview_active)" />
       <div id="loggerheadCont">
 
-         <div id="search_terms"></div>
+        <div id="search_terms"></div>
 
         <h1 metal:define-slot="heading"></h1>
         <div metal:define-slot="content"></div>
 
         <p class="fl">Loggerhead runs on <a href="http://bazaar-vcs.org/" class="link">Bazaar</a> branches</p>
-    </div>
+      </div>
   </body>
 </html>

=== added file 'loggerhead/templates/menu.pt'
--- loggerhead/templates/menu.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/menu.pt	2008-08-13 17:33:41 +0000
@@ -0,0 +1,19 @@
+<tal:menu>
+    <ul id="menuTabs">
+        <tal:changes-active tal:condition="not:fileview_active">
+        <li><a tal:attributes="href python:url('/changes', clear=1);
+                               title string:Changes;
+                               id string:on">Changes</a></li>
+        <li><a tal:attributes="href python:url('/files', clear=1);
+                               title string:Files">Files</a></li>
+        </tal:changes-active>
+        <tal:fileview-active tal:condition="fileview_active">
+        <li><a tal:attributes="href python:url('/changes', clear=1);
+                               title string:Changes">Changes</a></li>
+        <li><a tal:attributes="href python:url('/files', clear=1);
+                               title string:Files;
+                               id string:on">Files</a></li>
+        </tal:fileview-active>
+      </ul>
+</tal:menu>
+

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2008-08-04 03:42:01 +0000
+++ loggerhead/templates/revision.pt	2008-08-14 04:02:14 +0000
@@ -43,6 +43,12 @@
                 compare with revision <tal:b content="python:history.get_revno(remember)" />
                 </a>
             </li>
+            <li>
+            <a tal:condition="python:len(change.parents) > 0 and compare_revid is None"
+                   tal:attributes="href python:url(['/diff', change.revno], clear=1)">Download Diff</a>
+            <a tal:condition="python:compare_revid is not None"
+                   tal:attributes="href python:url(['/diff', change.revno, history.get_revno(compare_revid)], clear=1)">Download Diff</a>
+            </li>
             <li id="last"><a tal:attributes="href python:url(['/changes', change.revno]); 
                                   title string:view history from revision ${change/revno}"
                   tal:content="string:view history from revision ${change/revno}"></a></li>

=== added file 'loggerhead/templates/search-box.pt'
--- loggerhead/templates/search-box.pt	1970-01-01 00:00:00 +0000
+++ loggerhead/templates/search-box.pt	2008-08-13 17:24:10 +0000
@@ -0,0 +1,10 @@
+<tal:search-box>
+  <form tal:attributes="action python:branch.url('/changes',
+                            start_revid=getattr(navigation,
+                            'start_revid', None),
+                            file_id=getattr(navigation, 'file_id', None))">
+    <label>search:</label>
+    <input type="search" name="q" id="q" autocomplete="off" onblur="hide_search();" />
+  </form>
+</tal:search-box>
+

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2008-07-26 17:53:10 +0000
+++ loggerhead/util.py	2008-08-06 21:25:24 +0000
@@ -59,7 +59,10 @@
 
 
 def date_time(value):
-    return value.strftime('%Y-%m-%d %T')
+    if value is not None:
+        return value.strftime('%Y-%m-%d %T')
+    else:
+        return 'N/A'
 
 
 def _displaydate(date):

=== modified file 'start-loggerhead'
--- start-loggerhead	2008-08-06 18:33:20 +0000
+++ start-loggerhead	2008-08-11 20:09:44 +0000
@@ -29,22 +29,22 @@
 from paste.httpexceptions import make_middleware
 from paste.translogger import make_filter
 
-from loggerhead import daemon, release
+from loggerhead import daemon
 from loggerhead.apps.config import Root
 
 
 def make_handler(config, filename):
     roll = config.get('log.roll', 'never')
     if roll == 'daily':
-        h = logging.handlers.TimedRotatingFileHandler(filename, 'midnight', 0, 100)
+        h = logging.handlers.TimedRotatingFileHandler(filename, 'midnight', 1, 100)
     elif roll == 'weekly':
-        h = logging.handlers.TimedRotatingFileHandler(filename, 'W0', 0, 100)
+        h = logging.handlers.TimedRotatingFileHandler(filename, 'W0', 1, 100)
     else:
         h = logging.FileHandler(filename)
     return h
 
 
-def setup_logging(logdir, config, foreground):
+def setup_logging(log_folder, config, foreground):
     # i hate that stupid logging config format, so just set up logging here.
 
     if not os.path.exists(log_folder):
@@ -79,17 +79,17 @@
     default_pidfile = os.path.join(home, 'loggerhead.pid')
     default_configfile = os.path.join(home, 'loggerhead.conf')
     default_log_folder = os.path.join(home, 'logs')
-    parser = OptionParser(usage='usage: %prog [options]', version='%prog ' + release.version)
+    parser = OptionParser(usage='usage: %prog [options]', version='%prog')
     parser.add_option('-f', '--foreground', action='store_true', dest='foreground', default=False,
                       help="run in the foreground; don't daemonize")
     parser.add_option('-C', '--check', action='store_true', dest='check', default=False,
                       help="only start if not already running (useful for cron jobs)")
     parser.add_option('-p', '--pidfile', dest="pidfile", default=default_pidfile,
-                      help="override pidfile location")
+                      type=str, help="override pidfile location")
     parser.add_option('-c', '--config-file', dest="configfile", default=default_configfile,
-                      help="override configuration file location")
-    parser.add_option('-L', '--log-folder', dest="log-folder", default=default_log_folder,
-                      help="override log file directory")
+					  type=str, help="override configuration file location")
+    parser.add_option('-L', '--log-folder', dest="log_folder", default=default_log_folder,
+                      type=str, help="override log file directory")
     options, args = parser.parse_args()
     if len(args) > 0:
         parser.error('No filename arguments are used, only options.')
@@ -106,8 +106,6 @@
     if extra_path:
         sys.path.insert(0, extra_path)
 
-    #turbogears.update_config(configfile="dev.cfg", modulename="loggerhead.config")
-
     potential_overrides = [ ('server.socket_port', int),
                             ('server.webpath', str),
                             ('server.thread_pool', int),
@@ -121,7 +119,6 @@
         value = config.get(key, None)
         if value is not None:
             value = keytype(value)
-            #turbogears.config.update({ key: value })
 
     if not options.foreground:
         sys.stderr.write('\n')

=== modified file 'stop-loggerhead'
--- stop-loggerhead	2008-08-04 23:46:37 +0000
+++ stop-loggerhead	2008-08-06 19:38:14 +0000
@@ -17,14 +17,22 @@
 
 import os
 import sys
+from optparse import OptionParser
 
 home = os.path.realpath(os.path.dirname(__file__))
-pidfile = os.path.join(home, 'loggerhead.pid')
+default_pidfile = os.path.join(home, 'loggerhead.pid')
+parser = OptionParser(usage='usage: %prog [options]', version='%prog')
+parser.add_option('-p', '--pidfile', dest="pidfile", default=default_pidfile,
+				  help="override pidfile location")
+
+options, args = parser.parse_args()
+if len(args) > 0:
+	parser.error('No filename arguments are used, only options.')
 
 try:
-    f = open(pidfile, 'r')
+    f = open(options.pidfile, 'r')
 except IOError, e:
-    print 'No pid file found.'
+    print 'Pid file %s not found.' % options.pidfile
     sys.exit(1)
 
 pid = int(f.readline())

=== modified file 'stop-loggerhead.1'
--- stop-loggerhead.1	2008-07-26 14:09:59 +0000
+++ stop-loggerhead.1	2008-08-06 18:39:15 +0000
@@ -7,6 +7,10 @@
 Loggerhead is a web viewer for projects in bazaar. It can be used to navigate 
 a branch history, annotate files, view patches, perform searches, etc. It's 
 heavily based on bazaar-webserve, which is itself based on hgweb for Mercurial.
+.SH OPTIONS
+.TP
+\fB\-p\fR, \fB\-\-pidfile=PIDFILE\fR
+override pid file location
 .PP
 The stop-loggerhead command stops the currently running loggerhead server.
 .SH "LICENSE"

